package limax.auany;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;

import org.w3c.dom.Element;

import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

import limax.util.ConcurrentEnvironment;
import limax.util.ElementHelper;
import limax.util.Trace;
import limax.xmlconfig.ConfigParser;
import limax.xmlconfig.Service;

public class XmlConfig {
	public interface DataMXBean {
		String getHttpServerIP();

		int getHttpServerPort();
	}

	public static final class LoadConfig implements ConfigParser {
		@Override
		public void parse(Element self) throws Exception {
			ElementHelper eh = new ElementHelper(self);

			String httpserverip = eh.getString("http.server.ip");
			int httpserverport = eh.getInt("http.server.port", 0);
			Map<String, HttpHandler> httphandlers = new HashMap<>();

			if (!httpserverip.isEmpty() && httpserverport > 0) {
				httphandlers.put("/", new HttpServerHandler());
				Service.addRunAfterEngineStartTask(() -> {
					try {
						final InetSocketAddress sa = new InetSocketAddress(httpserverip, httpserverport);
						final HttpServer server = HttpServer.create(sa, 0);
						final ExecutorService executor = ConcurrentEnvironment.getInstance()
								.newThreadPool("limax.auany.httpserver", 0);
						server.setExecutor(executor);
						httphandlers.entrySet().forEach(e -> server.createContext(e.getKey(), e.getValue()));
						server.start();
						Service.addRunBeforeEngineStopTask(() -> server.stop(0));
					} catch (IOException e) {
						if (Trace.isErrorEnabled())
							Trace.error("start http ( httpserverip = " + httpserverip + " httpserverport = "
									+ httpserverport + ")", e);
					}
				});
				Service.JMXRegister(new DataMXBean() {
					@Override
					public String getHttpServerIP() {
						return httpserverip;
					}

					@Override
					public int getHttpServerPort() {
						return httpserverport;
					}
				}, "limax.auany:type=XmlConfig,name=httpserver");
			}

			PlatManager.initialize(self, httphandlers);
			Service.addRunAfterEngineStopTask(() -> PlatManager.unInitialize());
			ProviderKeyStore.initialize(self);
			PayManager.initialize(self, httphandlers);
			Service.addRunAfterEngineStopTask(() -> PayManager.unInitialize());
			Service.JMXRegister(new MBeanOperators(), "limax.auany:type=XmlConfig,name=userinfo");
		}

	}
}
