package limax.edb;

import java.io.IOException;

class Cursor {
	private final PageCache cache;
	private long chainIndex;
	private ChainPage chainPage;
	private int chainPos;
	private long dataAddr;

	Cursor(PageCache cache) {
		this.cache = cache;
	}

	boolean isFound() {
		return dataAddr != 0;
	}

	boolean hasChain() {
		return chainIndex != 0;
	}

	int getChainPos() {
		return chainPos;
	}

	long getDataAddr() {
		return dataAddr;
	}

	Cursor find(final byte[] key) {
		return chainPage.find(key, this);
	}

	Cursor leftMost() {
		return chainPage.leftMost(this);
	}

	Cursor rightMost() {
		return chainPage.rightMost(this);
	}

	Cursor findNext(final byte[] key) throws IOException {
		return chainPage.findNext(key, this);
	}

	Cursor findPrev(final byte[] key) throws IOException {
		return chainPage.findPrev(key, this);
	}

	Cursor next() throws IOException {
		return chainPage.next(this);
	}

	Cursor prev() throws IOException {
		return chainPage.prev(this);
	}

	byte[] getKey() throws IOException {
		return chainPage.getKey(chainPos);
	}

	void unlockChain() {
		cache.unlockChain(chainIndex);
		chainIndex = 0;
	}

	Page getPage() {
		return chainPage.getPage();
	}

	void insert(final byte[] key, long dataAddr) throws IOException {
		chainPage.insert(chainPos, key, dataAddr);
	}

	long remove() throws IOException {
		return chainPage.remove(chainPos);
	}

	void saveDataAddr(long dataAddr) {
		chainPage.saveDataAddr(chainPos, dataAddr);
	}

	Cursor reset() {
		unlockChain();
		chainPage = null;
		chainPos = 0;
		dataAddr = 0;
		return this;
	}

	Cursor set(long chainIndex) throws IOException {
		if (!cache.tryLockChain(chainIndex))
			return null;
		unlockChain();
		chainPage = cache.rLoad(this.chainIndex = chainIndex).asChainPage();
		return this;
	}

	Cursor set(int chainPos, long dataAddr) {
		this.chainPos = chainPos;
		this.dataAddr = dataAddr;
		return this;
	}
}