package limax.zdb;

import java.io.IOException;
import java.util.Arrays;

import limax.codec.Octets;
import limax.codec.OctetsStream;

class StorageEdb implements StorageEngine {
	private final LoggerEdb edb;
	private final String tableName;

	public StorageEdb(LoggerEdb edb, String tableName) {
		this.edb = edb;
		this.tableName = tableName;
		try {
			this.edb.getDatabase().addTable(new String[] { tableName });
		} catch (Exception e) {
			throw new XError(e);
		}
	}

	@Override
	public boolean exist(OctetsStream key) {
		byte[] tkey = key.capacity() == key.size() ? key.array() : Arrays.copyOf(key.array(), key.size());
		try {
			return edb.getDatabase().exist(tableName, tkey);
		} catch (Exception e) {
			throw new XError(e);
		}
	}

	@Override
	public OctetsStream find(OctetsStream key) {
		byte[] tkey = key.capacity() == key.size() ? key.array() : Arrays.copyOf(key.array(), key.size());
		try {
			byte[] tvalue = edb.getDatabase().find(tableName, tkey);
			if (null == tvalue)
				return null;
			return OctetsStream.wrap(Octets.wrap(tvalue));
		} catch (Exception e) {
			throw new XError(e);
		}
	}

	@Override
	public void walk(IWalk iw) {
		try {
			edb.getDatabase().walk(tableName, new Query(iw));
		} catch (IOException e) {
			throw new XError(e);
		}
	}

	@Override
	public void remove(OctetsStream key) {
		byte[] tkey = key.capacity() == key.size() ? key.array() : Arrays.copyOf(key.array(), key.size());
		try {
			edb.getDatabase().remove(tableName, tkey);
		} catch (IOException e) {
			throw new XError(e);
		}
	}

	@Override
	public void replace(OctetsStream key, OctetsStream value) {
		byte[] tkey = key.capacity() == key.size() ? key.array() : Arrays.copyOf(key.array(), key.size());
		byte[] tvalue = value.capacity() == value.size() ? value.array() : Arrays.copyOf(value.array(), value.size());
		try {
			edb.getDatabase().replace(tableName, tkey, tvalue);
		} catch (Exception e) {
			throw new XError(e);
		}
	}

	@Override
	public boolean insert(OctetsStream key, OctetsStream value) {
		byte[] tkey = key.capacity() == key.size() ? key.array() : Arrays.copyOf(key.array(), key.size());
		byte[] tvalue = value.capacity() == value.size() ? value.array() : Arrays.copyOf(value.array(), value.size());
		try {
			return edb.getDatabase().insert(tableName, tkey, tvalue);
		} catch (Exception e) {
			throw new XError(e);
		}
	}

	static class Query implements limax.edb.QueryData {
		private IWalk iw;

		Query(IWalk iw) {
			this.iw = iw;
		}

		@Override
		public boolean update(byte[] key, byte[] value) {
			return iw.onRecord(key, value);
		}
	}

}
